package tictactoecodingame;

import java.util.ArrayList;

/**
 * Cette classe représente un nœud dans l'arbre de recherche de l'algorithme Monte Carlo Tree Search (MCTS).
 * Chaque nœud représente une position possible dans le jeu de Tic Tac Toe.
 */
public class Noeud {
    double nbVictoire;      // Nombre de victoires obtenues à partir de cette position pour le joueur du nœud parent
    int nbJoue;             // Nombre de simulations effectuées à partir de cette position
    int joueur;             // Numéro du joueur qui doit jouer à partir de cette position
    Noeud pere;             // Nœud parent de ce nœud dans l'arbre de recherche
    ArrayList<Coup> coups;  // Liste des coups possibles à partir de cette position
    ArrayList<Noeud> fils;  // fils[n] est le nœud résultant du coup coups[n]

    /**
     * Constructeur pour un nœud avec un joueur spécifié et un nœud parent.
     *
     * @param _joueur Le numéro du joueur qui doit jouer à partir de cette position.
     * @param _pere Le nœud parent de ce nœud dans l'arbre de recherche.
     */
    public Noeud(int _joueur , Noeud _pere) {
        nbVictoire = nbJoue = 0;
        joueur = _joueur;
        pere = _pere;
        coups = new ArrayList<Coup>();
        fils = new ArrayList<Noeud>();
    }

    /**
     * Constructeur pour un nœud avec un joueur spécifié et sans nœud parent.
     *
     * @param _joueur Le numéro du joueur qui doit jouer à partir de cette position.
     */
    public Noeud( int  _joueur ) {
        this(_joueur , null);
    }

    /**
     * Cette méthode vérifie si ce nœud a des nœuds enfants dans l'arbre de recherche.
     *
     * @return Vrai si ce nœud a des nœuds enfants, faux sinon.
     */
    public boolean aDesFils() {
        return !fils.isEmpty();
    }
}